#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gobject
import getpass
from os.path import exists
from os.path import sep as SEP
from os.path import expanduser
from os.path import split
from os import mkdir

if exists("src") and exists("data") and exists("locale") and exists("beholderlib"):
    # Running from beholder file (not installed)
    GLOBAL_ICONS_PATH = SEP.join(__file__.split(SEP)[:-2]+['data', 'icons',''])
else:
    # Already installed
    GLOBAL_ICONS_PATH = SEP.join(['','usr','share','icons','beholder',''])

ITEM_ID = -1
ROOT_ID = 0
GROUP_ID = 1
NO_GROUP_ID = 2
TAG_ID = 3
NO_TAG_ID = 4

SMALL_SIZE_INT = 16
MEDIUM_SIZE_INT = 32
BIG_SIZE_INT = 64

SMALL_SIZE = str(SMALL_SIZE_INT) + 'x' + str(SMALL_SIZE_INT)
MEDIUM_SIZE = str(MEDIUM_SIZE_INT) + 'x' + str(MEDIUM_SIZE_INT)
BIG_SIZE = str(BIG_SIZE_INT) + 'x' + str(BIG_SIZE_INT)

DEFAULT_GROUP_ICON_NAME = 'default_group_icon.png'
DEFAULT_HOST_ICON_NAME = 'default_host_icon.png'

DEFAULT_USER = getpass.getuser()
HOME = expanduser('~') + SEP + '.beholder'
HOME_ICONS_PATH = HOME + SEP + 'images'

SMALL_ICONS_PATH = SEP.join([HOME_ICONS_PATH, SMALL_SIZE, ''])
MEDIUM_ICONS_PATH = SEP.join([HOME_ICONS_PATH, MEDIUM_SIZE, ''])
BIG_ICONS_PATH = SEP.join([HOME_ICONS_PATH, BIG_SIZE, ''])

for path in [HOME, HOME_ICONS_PATH, SMALL_ICONS_PATH, MEDIUM_ICONS_PATH, BIG_ICONS_PATH]:
    if not exists(path):
        mkdir(path)


CONN_PRETTY_NAME = {
                    'telnet':'Telnet',
                    'ssh1':'SSh v1',
                    'ssh2':'SSh v2',
                    'www':'WWW'
                    }
SUPPORTED_CONN = {
                  'telnet':'23',
                  'ssh1':'22',
                  'ssh2':'22',
                  'www':'80',
                  }
CONN_SINTAX = {
               'telnet':'telnet -l %(user)s %(host)s %(port)s',
               'ssh1':'ssh -1 %(user)s@%(host)s -p %(port)s',
               'ssh2':'ssh -2 %(user)s@%(host)s -p %(port)s',
               'www':None,
               }


class ConnectionType:
    def __init__(self, type=None, port=None, username=None):
        self.type = None
        self.port = None
        self.username = username
        self.set_conn_type(type)
        self.set_conn_port(port)

    def set_conn_type(self, type=None):
        if type in SUPPORTED_CONN:
            self.type = type
        else:
            self.type = 'telnet'

    def set_conn_port(self, port=None):
        if self.type is None:
            self.set_conn_type()

        if port is None:
            self.port = SUPPORTED_CONN[self.type]
        else:
            self.port = port

    def as_dict(self):
        conn_dict = {}
        conn_dict['type'] = self.type
        conn_dict['port'] = self.port
        conn_dict['username'] = self.username
        return conn_dict


class Host(gobject.GObject):
    __gsignals__ = {
                    'host-updated'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                   }
    def __init__(self, address="", hostname=None, icon=None, **args):
        gobject.GObject.__init__(self)
        self.signal_flag = True

        #### General informations
        self.__hostname = address if hostname is None else hostname
        self.__address = address
        self.__icon = None
        self.icon = icon
        #self.icon = 'default_host_icon.png' if icon is None or icon == '' else icon
        #self.group = args.get('group', None)
        self.__tags = args.get('tags', [])

        #### Protocol
        self.__connection = ConnectionType(**args.get('connection', {}))

        #### Canned Lines
        #self.canned_lines = args.get('canned_commands', [])

    @apply
    def address():
        def fset(self, new_address):
            self.__address = new_address
            self.emit_sig('host-updated')

        def fget(self):
            return self.__address

        return property(**locals())

    @apply
    def hostname():
        def fset(self, new_hostname):
            raise AttributeError, "hostname can't be set! It's the HostList key"

        def fget(self):
            return self.__hostname

        return property(**locals())

    @apply
    def tags():
        def fset(self, tags):
            self.__tags = tags[:]
            self.emit_sig('host-updated')

        def fget(self):
            return self.__tags

        return property(**locals())

    @apply
    def connection():
        def fset(self, connection_dic):
            self.__connection = ConnectionType(**connection_dic)
            self.emit_sig('host-updated')

        def fget(self):
            return self.__connection

        return property(**locals())

    @apply
    def icon():
        def fset(self, icon):
            if icon == DEFAULT_HOST_ICON_NAME:
                self.__icon = None
            else:
                self.__icon = icon
            self.emit_sig('host-updated')

        def fget(self):
            return self.__icon

        return property(**locals())

    def get_command(self):
        if CONN_SINTAX[self.connection.type] is None:
            return None
        
        if self.connection.username is None:
            return CONN_SINTAX[self.connection.type]%{'host':self.__address,
                                                      'port':self.connection.port,
                                                      'user':DEFAULT_USER}
        else:
            return CONN_SINTAX[self.connection.type]%{'host':self.__address,
                                                      'port':self.connection.port,
                                                      'user':self.connection.username}

    def get_small_icon(self):
        if self.icon == None:
            return GLOBAL_ICONS_PATH + SMALL_SIZE + SEP + DEFAULT_HOST_ICON_NAME
        else:
            return SMALL_ICONS_PATH + self.icon

    def get_medium_icon(self):
        if self.icon == None:
            return GLOBAL_ICONS_PATH + MEDIUM_SIZE + SEP + DEFAULT_HOST_ICON_NAME
        else:
            return MEDIUM_ICONS_PATH + self.icon

    def get_big_icon(self):
        if self.icon == None:
            return GLOBAL_ICONS_PATH + BIG_SIZE + SEP + DEFAULT_HOST_ICON_NAME
        else:
            return BIG_ICONS_PATH + self.icon

    def as_dict(self):
        dict_repr = {}
        dict_repr['address'] = self.address
        dict_repr['hostname'] = self.hostname
        dict_repr['connection'] = self.connection.as_dict()
        dict_repr['tags'] = self.tags
        dict_repr['icon'] = self.icon
        return dict_repr

    def update_by_dict(self, new_host_prop):
        self.signal_flag = False
        if 'address' in new_host_prop:
            self.address = new_host_prop['address']
        if 'tags' in new_host_prop:
            self.tags = new_host_prop['tags']
        if 'connection' in new_host_prop:
            self.connection = new_host_prop['connection']
        if 'icon' in new_host_prop:
            self.icon = new_host_prop['icon']
        self.signal_flag = True
        self.emit_sig('host-updated')

    def emit_sig(self, signal):
        if self.signal_flag:
            self.emit(signal)


class HostList(gobject.GObject):
    __gsignals__ = {
                    'host-updated'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                    'host-removed'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                    'host-added'    : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                   }
    def __init__(self, hosts_dic):
        gobject.GObject.__init__(self)
        self.contents = {}
        for hostname, host_dic in hosts_dic.items():
            new_host = Host(**host_dic)
            new_host.connect('host-updated', self.__on_host_update)
            self.contents[hostname] = new_host

    def get_by_hostname(self, hostname):
        return self.contents[hostname]

    def update(self, **args):
        new_host = Host(**args)
        new_host.connect('host-updated', self.__on_host_update)
        self.contents[new_host.hostname] = new_host
        self.emit('host-added', new_host)

    def remove(self, hostname):
        if hostname not in self.contents:
            return False

        removed_host = self.contents.pop(hostname)
        self.emit('host-removed', removed_host)
        return True

    def as_dict(self):
        dict_repr = {}
        for host in self:
            dict_repr[host.hostname] = host.as_dict()

        return dict_repr

    def __on_host_update(self, host):
        self.emit('host-updated', host)

    def __contains__(self, key):
        return key in self.contents

    def __getitem__(self, key):
        return self.contents.values()[key]

    def __len__(self):
        return len(self.contents)

